class InvalidFileError(Exception):
    pass


class LanguageError(Exception):
    pass


class ExtError(Exception):
    pass


class ShooterAPIError(Exception):
    pass


class ZimukuSearchingError(Exception):
    pass